import {
  createReactFlowElements,
  dragPill,
  dragPillStarted,
  dragPillStopped,
} from '@graphpolaris/querybuilder/usecases';
import {
  setQuerybuilderNodes,
  useAppDispatch,
  useQuerybuilderNodes,
} from '@graphpolaris/shared/data-access/store';
import { useMemo, useRef } from 'react';
import ReactFlow, {
  ReactFlowProvider,
  Background,
  Node,
  isNode,
} from 'react-flow-renderer';
import styles from './querybuilder.module.scss';
import ConnectionLine from './customFlowLines/connection';
import ConnectionDragLine from './customFlowLines/connectionDrag';
import AttributeRFPill from './customFlowPills/attributepill/attributepill';
import EntityRFPill from './customFlowPills/entitypill/entitypill';
import RelationRFPill from './customFlowPills/relationpill/relationpill';

const nodeTypes = {
  entity: EntityRFPill,
  relation: RelationRFPill,
  attribute: AttributeRFPill,
};
const edgeTypes = {
  connection: ConnectionLine,
};

const onLoad = (reactFlowInstance: any) => {
  setTimeout(() => reactFlowInstance.fitView(), 0);
};

const QueryBuilder = (props: {}) => {
  const nodes = useQuerybuilderNodes();
  const dispatch = useAppDispatch();
  const isDraggingPill = useRef(false);
  console.log('inputnodes', nodes);

  const elements = useMemo(() => createReactFlowElements(nodes), [nodes]);

  const onNodeDrag = (
    event: React.MouseEvent<Element, MouseEvent>,
    node: Node<any>
  ) => {
    // Get the node in the elements list to get the previous location
    const pNode = elements.find((e) => e.id == node.id);
    if (!(pNode && isNode(pNode))) return;
    // This is then used to calculate the delta position
    const dx = node.position.x - pNode.position.x;
    const dy = node.position.y - pNode.position.y;

    // Check if we started dragging, if so, call the drag started usecase
    if (!isDraggingPill.current) {
      dragPillStarted(node.id, nodes);
      isDraggingPill.current = true;
    }

    // Call the drag usecase
    dragPill(node.id, nodes, dx, dy, node.position);

    // Dispatch the new graphology object, so reactflow will get rerendered
    dispatch(setQuerybuilderNodes(nodes.export()));
  };
  const onNodeDragStop = (
    event: React.MouseEvent<Element, MouseEvent>,
    node: Node<any>
  ) => {
    isDraggingPill.current = false;

    // Call the drag pill stopped usecase
    dragPillStopped(node.id, nodes);

    // Dispatch the new graphology object, so reactflow will get rerendered
    dispatch(setQuerybuilderNodes(nodes.export()));
  };

  return (
    <div>
      <ReactFlowProvider>
        <ReactFlow
          elements={elements}
          snapGrid={[10, 10]}
          // snapToGrid
          nodeTypes={nodeTypes}
          edgeTypes={edgeTypes}
          connectionLineComponent={ConnectionDragLine}
          onLoad={onLoad}
          onNodeDrag={onNodeDrag}
          onNodeDragStop={onNodeDragStop}
          className={styles.reactflow}
        >
          <Background gap={10} size={0.7} />
        </ReactFlow>
      </ReactFlowProvider>
    </div>
  );
};

export default QueryBuilder;
